@@ -7,7 +7,8 @@ Module for handling linelist data from the VALD3 database (http://vald.astro.uu.
 import logging
 import re
 from io import StringIO
-from os.path import dirname, join
+from os.path import dirname, join, exists
+from copy import deepcopy
 
 import numpy as np
 import pandas as pd
@@ -182,13 +183,16 @@ class ValdFile(LineList):
 
         # Determine File type and format
         valdtype, fmt = self.identify_valdtype(lines)
+        self.valdtype = valdtype
 
         # Determine the number of lines in the file
         if valdtype == "extract_stellar":
             n = self.parse_header(lines[0])
             # Skip the info header if extract stellar
+            self.header = lines[0]
             lines = lines[1:]
         else:
+            self.header = ''
             n = self.parse_nlines(lines, fmt, valdtype)
 
         # Determine the units and medium in the linelist
@@ -199,15 +203,19 @@ class ValdFile(LineList):
             if fmt == "long":
                 linedata = lines[2 : 2 + n * 4]
                 refdata = linedata[3::4]
+                self._ref_record = lines[2 + n * 4:]
                 if valdtype == "extract_stellar":
                     atmodata = lines[2 + n * 4]
                     abunddata = lines[3 + n * 4 : 21 + n * 4]
+                    self._ref_record = lines[21 + n * 4:]
             elif fmt == "short":
                 linedata = lines[2 : 2 + n]
                 refdata = linedata
+                self._ref_record = lines[2 + n:]
                 if valdtype == "extract_stellar":
                     atmodata = lines[2 + n]
                     abunddata = lines[3 + n : 21 + n]
+                    self._ref_record = lines[21 + n:]
         except IndexError:
             msg = "Linelist file is shorter than it should be according to the number of lines. Is it incomplete?"
             logger.error(msg)
@@ -326,7 +334,20 @@ class ValdFile(LineList):
         """
 
         if fmt == "short":
-            names = [
+            if valdtype == "extract_all":
+                names = [
+                    "species",
+                    "wlcent",
+                    "excit",
+                    "gflog",
+                    "gamrad",
+                    "gamqst",
+                    "gamvw",
+                    "lande",
+                    "reference",
+                ]
+            if valdtype == "extract_stellar":
+                names = [
                 "species",
                 "wlcent",
                 "excit",
@@ -336,10 +357,9 @@ class ValdFile(LineList):
                 "gamqst",
                 "gamvw",
                 "lande",
+                "depth",
                 "reference",
             ]
-            if valdtype == "extract_stellar":
-                names = names[:-1] + ["depth", names[-1]]
 
         elif fmt == "long":
             names = [
@@ -391,11 +411,17 @@ class ValdFile(LineList):
             # Extract Stellar has quotation marks around the levels
             # extract element does not...
             if valdtype == "extract_stellar":
+                couple_lower = [t[1:8].strip() for t in term_lower]
                 term_lower = [t.strip()[8:-1].strip() for t in term_lower]
+                couple_upper = [t[1:8].strip() for t in term_upper]
                 term_upper = [t.strip()[8:-1].strip() for t in term_upper]
             else:
+                couple_lower = [t[:8].strip() for t in term_lower]
                 term_lower = [t.strip()[8:].strip() for t in term_lower]
+                couple_upper = [t[:8].strip() for t in term_upper]
                 term_upper = [t.strip()[8:].strip() for t in term_upper]
+            couple_lower = [ele  if ele != '' else '  ' for ele in couple_lower]
+            couple_upper = [ele  if ele != '' else '  ' for ele in couple_upper]
 
             term_lower = np.char.partition(term_lower, " ")[:, (0, 2)]
             term_lower = np.char.strip(term_lower)
@@ -413,7 +439,9 @@ class ValdFile(LineList):
                 np.char.add(term_upper[:, 0], " "), term_upper[:, 1]
             )
 
+            linelist["couple_lower"] = couple_lower
             linelist["term_lower"] = term_lower
+            linelist["couple_upper"] = couple_upper
             linelist["term_upper"] = term_upper
 
             # extract error data
@@ -531,3 +559,177 @@ class ValdFile(LineList):
                 logger.debug(ex)
         bibdata_filtered = pybtex.database.BibliographyData(entries)
         return bibdata_filtered.to_string("bibtex")
+
+    def save(self, filename, overwrite=False):
+        '''
+        Save the line list to a file, in VALD format.
+        '''
+
+        list_save_content = []
+        # Output the header
+        if self.header != '':
+            list_save_content.append(self.header)
+        
+        # 2. Output each line in wavelength 
+        if self.lineformat == 'long':
+            if self.valdtype == 'extract_stellar':
+                line_header = '                                                                     Lande factors       Damping parameters  Central\nSpec Ion       WL_air(A)  log gf* E_low(eV) J lo E_up(eV)  J up  lower   upper    mean   Rad.   Stark  Waals  depth'
+            elif self.valdtype == 'extract_all':
+                line_header = '''                                                                     Lande factors        Damping parameters\nElm Ion       WL_air(A)  log gf* E_low(eV) J lo  E_up(eV) J up   lower   upper    mean   Rad.  Stark    Waals'''
+        elif self.lineformat == 'short':
+            if self.valdtype == 'extract_stellar':
+                line_header = '''                                                 Damping parameters   Lande  Central\nSpec Ion       WL_air(A) Excit(eV) Vmic log gf*  Rad.   Stark  Waals  factor  depth  Reference'''
+            elif self.valdtype == 'extract_all':
+                line_header = '''                                             Damping parameters   Lande\nElm Ion       WL_air(A) Excit(eV) log gf*   Rad.  Stark    Waals  factor   References'''
+        else:
+            raise ValueError('VALD line format not recognized.')
+        list_save_content.append(line_header)
+
+        self._lines['species_len'] = self._lines['species'].apply(len)
+        if self.lineformat == 'long':
+            if self.valdtype == 'extract_stellar':
+                def line_to_text(row):
+                    return f"'{row['species']}',{row['wlcent']:{21-row['species_len']}.5f},{row['gflog']:7.3f},{row['excit']:8.4f},{row['j_lo']:5.1f},{row['e_upp']:8.4f},{row['j_up']:5.1f},{row['lande_lower']:7.3f},{row['lande_upper']:7.3f},{row['lande']:7.3f},{row['gamrad']:6.3f},{row['gamqst']:6.3f},{row['gamvw']:6.3f},{row['depth']:6.3f},\n'  {row['couple_lower']}   {row['term_lower']:>83}'\n'  {row['couple_upper']}   {row['term_upper']:>83}'\n'{row['reference']}'" 
+            elif self.valdtype == 'extract_all':
+                def line_to_text(row):
+                    return f"'{row['species']}',{row['wlcent']:{20-row['species_len']}.5f},{row['gflog']:8.3f},{row['excit']:8.4f},{row['j_lo']:5.1f},{row['e_upp']:8.4f},{row['j_up']:5.1f},{row['lande_lower']:7.3f},{row['lande_upper']:7.3f},{row['lande']:7.3f},{row['gamrad']:6.3f},{row['gamqst']:6.3f},{row['gamvw']:6.3f},\n  {row['couple_lower']}   {row['term_lower']:>83}\n  {row['couple_upper']}   {row['term_upper']:>83}\n'{row['reference']}'" 
+        elif self.lineformat == 'short':
+            if self.valdtype == 'extract_stellar':
+                def line_to_text(row):
+                    return f"'{row['species']}',{row['wlcent']:{21-row['species_len']}.5f},{row['excit']:8.4f},{row['vmic']:4.1f},{row['gflog']:7.3f},{row['gamrad']:6.3f},{row['gamqst']:6.3f},{row['gamvw']:6.3f},{row['lande']:7.3f},{row['depth']:6.3f}, '{row['reference']}'" 
+            elif self.valdtype == 'extract_all':
+                def line_to_text(row):
+                    return f"'{row['species']}',{row['wlcent']:{20-row['species_len']}.5f},{row['excit']:9.3f},{row['gflog']:7.3f},{row['gamrad']:6.3f},{row['gamqst']:6.3f},{row['gamvw']:6.3f},{row['lande']:7.3f},'{row['reference']}'" 
+        line_text = list(self._lines.apply(line_to_text, axis=1).values)
+        self._lines = self._lines.drop('species_len', axis=1)
+        list_save_content += line_text
+        
+        # Only for extract_stellar: add model and abund
+        if self.valdtype == 'extract_stellar':
+            list_save_content.append(f"'{self.atmo}',")
+            pattern = self.abund.totype(self.abund.pattern, 'H=12')
+            abund_text = ''
+            count = 1
+            for ele in pattern.keys():
+                abund_text += f"'{ele:<2}:{pattern[ele]:6.2f}',"
+                if ele == 'He' or (count-2) % 6 == 0:
+                    abund_text += '\n'
+                count += 1
+            abund_text += "'END'"
+            list_save_content += [abund_text]
+        
+        # 4. Output references
+        list_save_content += self._ref_record
+        list_save_content = [ele if ele[-1:] == '\n' else ele+'\n' for ele in list_save_content]
+
+        if exists(filename) and not overwrite:
+            raise FileExistsError(f"The file '{filename}' already exists. Use overwrite=True to overwrite it.")
+        else:
+            # 如果文件不存在，直接写入
+            with open(filename, 'w') as file:
+                file.writelines(list_save_content)
+
+    @staticmethod
+    def _get_ref_pair(vlist):
+        '''
+        Get the reference tag, description and number pairs.
+        '''
+        if vlist.lineformat == 'long':
+            ref_pair = [ele[29:].replace('iso:', '').replace('wl:', '').replace('gf:', '').split()[:-1] if 'hfs:' not in ele else ele[29:].replace('iso:', '').replace('wl:', '').replace('gf:', '').split()[:-2] for ele in vlist['reference']]
+        elif vlist.lineformat == 'short':
+            ref_pair = [ele.replace('iso:', '').replace('wl:', '').replace('gf:', '').split()[:-1] if 'hfs:' not in ele else ele.replace('iso:', '').replace('wl:', '').replace('gf:', '').split()[:-2] for ele in vlist['reference']]
+        ref_pair = [item for sublist in ref_pair for item in sublist]
+        pairs = [(ref_pair[i+1]+'|'+ref_pair[i], int(ref_pair[i])) for i in range(0, len(ref_pair), 2)]
+        # return pairs
+        ref_pair = list(set(pairs))
+        ref_pair = sorted(ref_pair, key=lambda x: x[1])
+        
+        ref_pair_dict = {}
+        ref_record_only = [ele[5:] for ele in vlist._ref_record[2:]]
+        for ele in ref_pair:
+            ref_pair_dict[ele[0].split('|')[0] + '|' + ref_record_only[ele[1]-1]] = [ref_record_only[ele[1]-1], ele[1]]
+
+        return ref_pair_dict
+
+    @staticmethod
+    def _merge_ref_pair(ref_pair_dict_1, ref_pair_dict_2):
+        '''
+        Merge two ref_pair
+        '''
+        new_count = len(ref_pair_dict_1) + 1
+        merge_ref_pair_dict = ref_pair_dict_1.copy()
+        for key in ref_pair_dict_2.keys():
+            if key in ref_pair_dict_1.keys():
+                ref_pair_dict_2[key].append(ref_pair_dict_1[key][-1])
+            else:
+                merge_ref_pair_dict[key+'_list2'] = [ref_pair_dict_2[key][0], new_count]
+                ref_pair_dict_2[key].append(new_count)
+                new_count += 1
+        return merge_ref_pair_dict, ref_pair_dict_2
+
+    @staticmethod
+    def _renew_ref_number(ref_string, ref_pair, lineformat):
+        if lineformat == 'long':
+            t = [ref_string[:29], ref_string[29:].split()]
+        elif lineformat == 'short':
+            t = [' ', ref_string.split()]
+        for i in range(0, len(t[1]), 2):
+            if i <= len(t[1])-3:
+                try:
+                    t[1][i] = str(ref_pair[t[1][i+1].replace('iso:', '').replace('wl:', '').replace('gf:', '') + '-' + t[1][i]][-1])
+                except:
+                    t[1][i] = '00'
+        t[1] = ' '.join(t[1])
+        t = ' '.join(t)
+        return t
+
+    @staticmethod
+    def merge_list(vlist_1, vlist_2):
+        '''
+        Combine two VALD line list. The two line list must have the same short/long format.
+        Note that the code will use the metadata from vlist_1 as the ones in the combined line list.
+        Lines with same 'species', 'wlcent', 'gflog' and 'excit' will be treated as duplicated lines and removed.
+        Note: reference mismatch is known to be happen during line list merge.
+        '''
+
+        # Check the format of the line lists.
+        if vlist_1.lineformat != vlist_2.lineformat:
+            raise ValueError('lineformat of the line lists not the same.')
+        if vlist_1.medium != vlist_2.medium:
+            raise ValueError('medium of the line lists not the same.')
+        if vlist_1.unit != vlist_2.unit:
+            raise ValueError('unit of the line lists not the same.')
+
+        vlist_1_use, vlist_2_use = deepcopy(vlist_1), deepcopy(vlist_2)
+
+        ref_pair_1 = ValdFile._get_ref_pair(vlist_1_use)
+        ref_pair_2 = ValdFile._get_ref_pair(vlist_2_use)
+        ref_pair_1, ref_pair_2 = ValdFile._merge_ref_pair(ref_pair_1, ref_pair_2)
+        ref_pair_2_final = {}
+        for key in ref_pair_2.keys():
+            ref_pair_2_final[key.split('|')[0]+'-'+str(ref_pair_2[key][1])] = ref_pair_2[key]
+
+        # Replace the reference numbers in vlist_2
+        vlist_2_use._lines['reference'] = vlist_2_use._lines['reference'].apply(ValdFile._renew_ref_number, ref_pair=ref_pair_2_final, lineformat=vlist_2_use.lineformat)
+
+        ref_record_combined = []
+        count = 1
+        num_length = len(str(len(ref_pair_1)))
+        for key in ref_pair_1.keys():
+            ref_record_combined.append(f"{count:{num_length}.0f}. {ref_pair_1[key][0]}")
+            count += 1
+        
+        vlist_1_use._ref_record = vlist_1_use._ref_record[:2] + ref_record_combined
+        
+        # Concat the two dfs
+        vlist_1_use._lines = pd.concat([vlist_1_use._lines, vlist_2_use._lines])
+        
+        vlist_1_use._lines = vlist_1_use._lines[~vlist_1_use._lines.duplicated(subset=['species', 'wlcent', 'gflog', 'excit'], keep='first')].sort_values('wlcent').reset_index(drop=True)
+        vlist_1_use.nlines = len(vlist_1_use)
+
+        if vlist_1_use.valdtype == 'extract_stellar':
+            header_split = vlist_1_use.header.split(',')
+            header_split[2] = f' {len(vlist_1_use)}'
+            vlist_1_use.header = ','.join(header_split)
+        
+        return vlist_1_use
\ No newline at end of file
