@@ -28,6 +28,9 @@ from .sme import MASK_VALUES
 from .synthesize import Synthesizer
 from .util import print_to_log, show_progress_bars
 
+# # Debug usage
+# from memory_profiler import profile
+
 logger = logging.getLogger(__name__)
 
 clight = speed_of_light * 1e-3  # km/s
@@ -175,7 +178,15 @@ class SME_Solver:
 
         # change parameters
         for name, value in zip(self.parameter_names, param):
+            if self.dynamic_param is not None:
+                if name in self.dynamic_param.keys():
+                    raise ValueError(f'fitting parameter {name} cannot be also in dynamic parameter.')
             sme[name] = value
+        # change dynamic parameters
+        if self.dynamic_param is not None:
+            for name in self.dynamic_param.keys():
+                sme[name] = self.dynamic_param[name](sme)
+                print(f'Changing dynamic parameter {name} to {sme[name]:.2f}.')
         # run spectral synthesis
         try:
             result = self.synthesizer.synthesize_spectrum(
@@ -596,7 +607,7 @@ class SME_Solver:
                 )
         return param_names
 
-    def solve(self, sme, param_names=None, segments="all", bounds=None):
+    def solve(self, sme, param_names=None, segments="all", bounds=None, step_sizes=None, dynamic_param=None):
         """
         Find the least squares fit parameters to an observed spectrum
 
@@ -620,6 +631,8 @@ class SME_Solver:
         assert "wave" in sme, "SME Structure has no wavelength"
         assert "spec" in sme, "SME Structure has no observation"
 
+        self.dynamic_param = dynamic_param
+
         if self.restore and self.filename is not None:
             fname = self.filename.rsplit(".", 1)[0]
             fname = f"{fname}_iter.json"
@@ -670,7 +683,8 @@ class SME_Solver:
         if bounds is None:
             bounds = self.get_bounds(sme)
         # scales = self.get_scale()
-        step_sizes = self.get_step_sizes(self.parameter_names)
+        if step_sizes is None:
+            step_sizes = self.get_step_sizes(self.parameter_names)
         # Starting values
         p0 = self.get_default_values(sme)
         if np.any((p0 < bounds[0]) | (p0 > bounds[1])):
@@ -715,7 +729,7 @@ class SME_Solver:
         # Setup LineList only once
         dll = self.synthesizer.get_dll()
         dll.SetLibraryPath()
-        dll.InputLineList(sme.linelist)
+        _ = dll.InputLineList(sme.linelist)
 
         # Do the heavy lifting
         if self.nparam > 0:
@@ -764,7 +778,7 @@ class SME_Solver:
                     self.parameter_names, res.x, sme.fitresults.fit_uncertainties
                 ):
                     logger.info("%s\t%.5f +- %.5g", name.ljust(10), value, unc)
-                logger.info("%s\t%s +- %s", "v_rad".ljust(10), sme.vrad, sme.vrad_unc)
+                logger.info("%s\t%s +- %s", "vrad".ljust(10), sme.vrad, sme.vrad_unc)
             except:
                 pass
         elif len(param_names) > 0:
