@@ -8,7 +8,7 @@ import uuid
 import numpy as np
 from scipy.constants import speed_of_light
 from scipy.interpolate import interp1d
-from scipy.ndimage.filters import convolve
+from scipy.ndimage import convolve
 from tqdm import tqdm
 
 from . import broadening
@@ -24,6 +24,10 @@ from .sme import MASK_VALUES
 from .sme_synth import SME_DLL
 from .util import show_progress_bars
 
+# Temp solution
+import pandas as pd
+pd.options.mode.chained_assignment = None  # None means no warning will be shown
+
 logger = logging.getLogger(__name__)
 
 clight = speed_of_light * 1e-3  # km/s
@@ -132,9 +136,10 @@ class Synthesizer:
         diff = wint[1:] - wint[:-1]
         jmin = np.argmin(diff)
         vstep1 = diff[jmin] / wint[jmin] * clight  # smallest step
-        vstep2 = 0.1 * wspan / (len(wint) - 1) / wmid * clight  # 10% mean dispersion
+        # vstep2 = 0.1 * wspan / (len(wint) - 1) / wmid * clight  # 10% mean dispersion
+        # vstep2 = np.median(diff / wint[:-1] * clight) # Median step
         vstep3 = 0.05  # 0.05 km/s step
-        vstep = max(vstep1, vstep2, vstep3)  # select the largest
+        vstep = np.max([vstep1, vstep3])  # select the largest
 
         # Generate model wavelength scale X, with uniform wavelength step.
         nx = int(
@@ -470,6 +475,7 @@ class Synthesizer:
         sme,
         segments="all",
         passLineList=True,
+        linelist_mode='all',
         passAtmosphere=True,
         passNLTE=True,
         updateStructure=True,
@@ -544,6 +550,8 @@ class Synthesizer:
         smod = [[] for _ in range(n_segments)]
         cmod = [[] for _ in range(n_segments)]
         wmod = [[] for _ in range(n_segments)]
+        central_depth = [[] for _ in range(n_segments)]
+        line_range = [[] for _ in range(n_segments)]
 
         # If wavelengths are already defined use those as output
         if "wave" in sme:
@@ -554,7 +562,8 @@ class Synthesizer:
         # Input Model data to C library
         dll.SetLibraryPath()
         if passLineList:
-            dll.InputLineList(sme.linelist)
+            line_ion_mask = dll.InputLineList(sme.linelist)
+            sme.line_ion_mask = line_ion_mask
         if hasattr(updateLineList, "__len__") and len(updateLineList) > 0:
             # TODO Currently Updates the whole linelist, could be improved to only change affected lines
             dll.UpdateLineList(sme.atomic, sme.species, updateLineList)
@@ -574,15 +583,13 @@ class Synthesizer:
         #   Interpolate onto geomspaced wavelength grid
         #   Apply instrumental and turbulence broadening
 
-        for il in tqdm(
-            segments, desc="Segments", leave=False, disable=~show_progress_bars
-        ):
-            wmod[il], smod[il], cmod[il] = self.synthesize_segment(
+        for il in tqdm(segments, desc="Segments", leave=False, disable=~show_progress_bars):
+            wmod[il], smod[il], cmod[il], central_depth[il], line_range[il] = self.synthesize_segment(
                 sme,
                 il,
                 reuse_wavelength_grid,
                 il != segments[0],
-                dll_id=dll_id,
+                dll_id=dll_id
             )
 
         for il in segments:
@@ -594,7 +601,12 @@ class Synthesizer:
                 wave[il] = np.concatenate(([wbeg], wmod[il][itrim], [wend]))
 
         if sme.specific_intensities_only:
-            return wmod, smod, cmod
+            return wmod, smod, cmod, central_depth
+
+        # For testing wavegrid
+        sme.wmod = wmod.copy()
+        sme.smod = smod.copy()
+        sme.comd = cmod.copy()
 
         # Fit continuum and radial velocity
         # And interpolate the flux onto the wavelength grid
@@ -631,11 +643,23 @@ class Synthesizer:
                 sme.synth = smod
             if "cont" not in sme:
                 sme.cont = cmod
+            if "line_range" not in sme:
+                sme.line_range = line_range
+            
+            if passLineList:
+                sme.central_depth = central_depth
+                for s in segments:
+                    # To do: modify to let linelist also can accept segments.
+                    if len(central_depth[s] > 0):
+                        sme.linelist._lines.loc[~line_ion_mask, 'central_depth'] = central_depth[s]
+                        sme.linelist._lines.loc[line_ion_mask, 'central_depth'] = np.nan
 
             for s in segments:
                 sme.wave[s] = wave[s]
                 sme.synth[s] = smod[s]
                 sme.cont[s] = cmod[s]
+                # sme.central_depth[s] = central_depth[s]
+                sme.line_range[s] = line_range[s]
 
             if sme.cscale_type in ["spline", "spline+mask"]:
                 sme.cscale = np.asarray(cscale)
@@ -650,6 +674,7 @@ class Synthesizer:
             sme.vrad = np.asarray(vrad)
             sme.vrad_unc = np.asarray(vrad_unc)
             sme.nlte.flags = dll.GetNLTEflags()
+
             result = sme
         else:
             wave = Iliffe_vector(values=wave)
@@ -667,6 +692,8 @@ class Synthesizer:
         reuse_wavelength_grid=False,
         keep_line_opacity=False,
         dll_id=None,
+        passLineList=True,
+        linelist_mode='all'
     ):
         """Create the synthetic spectrum of a single segment
 
@@ -676,6 +703,8 @@ class Synthesizer:
             The SME strcuture containing all relevant parameters
         segment : int
             the segment to synthesize
+        setLineList : bool, optional
+            wether to pass the linelist to the c library (default: True)
         reuse_wavelength_grid : bool
             Whether to keep the current wavelength grid for the synthesis
             or create a new one, depending on the linelist. Default: False
@@ -699,6 +728,15 @@ class Synthesizer:
         vrad_seg = sme.vrad[segment] if sme.vrad[segment] is not None else 0
         wbeg, wend = self.get_wavelengthrange(sme.wran[segment], vrad_seg, sme.vsini)
 
+        # # Input the line list as required
+        # if passLineList:
+        #     print('Pass line list.')
+        #     if linelist_mode == 'chunk':
+        #         print('Linelist: chunk')
+        #         print()
+        #         indices = (sme.linelist['wlcent'] >= wbeg) & (sme.linelist['wlcent'] <= wend)
+        #         dll.InputLineList(sme.linelist[indices])
+
         dll.InputWaveRange(wbeg, wend)
         dll.Opacity()
 
@@ -748,17 +786,19 @@ class Synthesizer:
             if "iptype" in sme:
                 logger.debug("Apply detector broadening")
                 ipres = sme.ipres if np.size(sme.ipres) == 1 else sme.ipres[segment]
-                sint = broadening.apply_broadening(
-                    ipres, wint, sint, type=sme.iptype, sme=sme
-                )
+                sint = broadening.apply_broadening(ipres, wint, sint, type=sme.iptype, sme=sme)
 
         # Divide calculated spectrum by continuum
         if sme.normalize_by_continuum:
             sint /= cint
 
-        return wint, sint, cint
+        # Mingjie: run CentralDepth
+        central_depth = dll.CentralDepth(sme.mu, sme.accrt)
+        line_range = dll.GetLineRange()
+
+        return wint, sint, cint, central_depth, line_range
 
 
-def synthesize_spectrum(sme, segments="all"):
+def synthesize_spectrum(sme, segments="all", **args):
     synthesizer = Synthesizer()
-    return synthesizer.synthesize_spectrum(sme, segments)
+    return synthesizer.synthesize_spectrum(sme, segments, **args)
